@@ -314,7 +314,14 @@ class SeedConfiguration(ConfigurationBase):
                         self.refresh_timestamp = 0
 
                 md = dict(name=self.name, cache_name=cache_name, grid_name=grid_name)
-                yield SeedTask(md, tile_manager, levels, self.refresh_timestamp, coverage)
+
+                if tile_manager.rescale_tiles:
+                    if tile_manager.rescale_tiles > 0:
+                        levels = levels[::-1]
+                    for l in levels:
+                        yield SeedTask(md, tile_manager, [l], self.refresh_timestamp, coverage)
+                else:
+                    yield SeedTask(md, tile_manager, levels, self.refresh_timestamp, coverage)
 
 class CleanupConfiguration(ConfigurationBase):
     def __init__(self, name, conf, seeding_conf):
