@@ -19,7 +19,9 @@ WMS service handler
 from mapproxy.compat import iteritems
 from mapproxy.compat.itertools import chain
 from functools import partial
+from math import sqrt
 from mapproxy.cache.tile import CacheInfo
+from mapproxy.featureinfo import combine_docs
 from mapproxy.request.wms import (wms_request, WMS111LegendGraphicRequest,
     mimetype_from_infotype, infotype_from_mimetype, switch_bbox_epsg_axis_order)
 from mapproxy.srs import SRS, TransformationError
@@ -27,13 +29,13 @@ from mapproxy.service.base import Server
 from mapproxy.response import Response
 from mapproxy.source import SourceError
 from mapproxy.exception import RequestError
-from mapproxy.image import bbox_position_in_image, SubImageSource, BlankImageSource
+from mapproxy.image import bbox_position_in_image, SubImageSource, BlankImageSource, GeoReference
 from mapproxy.image.merge import concat_legends, LayerMerger
 from mapproxy.image.opts import ImageOptions
 from mapproxy.image.message import attribution_image, message_image
 from mapproxy.layer import BlankImage, MapQuery, InfoQuery, LegendQuery, MapError, LimitedLayer
 from mapproxy.layer import MapBBOXError, merge_layer_extents, merge_layer_res_ranges
-from mapproxy.util import async
+from mapproxy.util import async_
 from mapproxy.util.py import cached_property, reraise
 from mapproxy.util.coverage import load_limited_to
 from mapproxy.util.ext.odict import odict
@@ -150,6 +152,7 @@ class WMSServer(Server):
             map_request.http.environ, (query.srs.srs_code, query.bbox))
 
         try:
+            result.georef = GeoReference(bbox=orig_query.bbox, srs=orig_query.srs)
             result_buf = result.as_buffer(img_opts)
         except IOError as ex:
             raise RequestError('error while processing image file: %s' % ex,
@@ -192,7 +195,7 @@ class WMSServer(Server):
         info_formats = [mimetype_from_infotype(map_request.version, info_type) for info_type in info_types]
         result = Capabilities(service, root_layer, tile_layers,
             self.image_formats, info_formats, srs=self.srs, srs_extents=self.srs_extents,
-            inspire_md=self.inspire_md,
+            inspire_md=self.inspire_md, max_output_pixels=self.max_output_pixels
             ).render(map_request)
         return Response(result, mimetype=map_request.mime_type)
 
@@ -240,28 +243,22 @@ class WMSServer(Server):
             return Response('', mimetype=mimetype)
 
         if self.fi_transformers:
-            doc = infos[0].combine(infos)
-            if doc.info_type == 'text':
-                resp = doc.as_string()
-                mimetype = 'text/plain'
-            else:
-                if not mimetype:
-                    if 'xml' in self.fi_transformers:
-                        info_type = 'xml'
-                    elif 'html' in self.fi_transformers:
-                        info_type = 'html'
-                    else:
-                        info_type = 'text'
-                    mimetype = mimetype_from_infotype(request.version, info_type)
+            if not mimetype:
+                if 'xml' in self.fi_transformers:
+                    info_type = 'xml'
+                elif 'html' in self.fi_transformers:
+                    info_type = 'html'
                 else:
-                    info_type = infotype_from_mimetype(request.version, mimetype)
-                resp = self.fi_transformers[info_type](doc).as_string()
-        else:
-            mimetype = mimetype_from_infotype(request.version, infos[0].info_type)
-            if len(infos) > 1:
-                resp = infos[0].combine(infos).as_string()
+                    info_type = 'text'
+                mimetype = mimetype_from_infotype(request.version, info_type)
             else:
-                resp = infos[0].as_string()
+                info_type = infotype_from_mimetype(request.version, mimetype)
+            resp, actual_info_type = combine_docs(infos, self.fi_transformers[info_type])
+            if actual_info_type is not None and info_type != actual_info_type:
+                mimetype = mimetype_from_infotype(request.version, actual_info_type)
+        else:
+            resp, info_type = combine_docs(infos)
+            mimetype = mimetype_from_infotype(request.version, info_type)
 
         return Response(resp, mimetype=mimetype)
 
@@ -468,7 +465,7 @@ class Capabilities(object):
     """
     def __init__(self, server_md, layers, tile_layers, image_formats, info_formats,
         srs, srs_extents=None, epsg_axis_order=False,
-        inspire_md=None,
+        inspire_md=None, max_output_pixels=None
         ):
         self.service = server_md
         self.layers = layers
@@ -478,9 +475,13 @@ class Capabilities(object):
         self.srs = srs
         self.srs_extents = limit_srs_extents(srs_extents, srs)
         self.inspire_md = inspire_md
+        self.max_output_pixels = max_output_pixels
 
     def layer_srs_bbox(self, layer, epsg_axis_order=False):
         for srs, extent in iteritems(self.srs_extents):
+            if srs not in self.srs:
+                continue
+
             # is_default is True when no explicit bbox is defined for this srs
             # use layer extent
             if extent.is_default:
@@ -489,13 +490,15 @@ class Capabilities(object):
                 bbox = extent.bbox_for(SRS(srs))
             else:
                 # Use intersection of srs_extent and layer.extent.
-                bbox = extent.intersection(layer.extent).bbox_for(SRS(srs))
+                # Use 4326 extents to avoid transformation errors.
+                a = extent.transform(SRS(4326))
+                b = layer.extent.transform(SRS(4326))
+                bbox = a.intersection(b).bbox_for(SRS(srs))
 
             if epsg_axis_order:
                 bbox = switch_bbox_epsg_axis_order(bbox, srs)
 
-            if srs in self.srs:
-                yield srs, bbox
+            yield srs, bbox
 
         # add native srs
         layer_srs_code = layer.extent.srs.srs_code
@@ -520,6 +523,12 @@ class Capabilities(object):
         inspire_md = None
         if self.inspire_md:
             inspire_md = recursive_bunch(default='', **self.inspire_md)
+
+        max_output_size = None
+        if self.max_output_pixels:
+            output_width = output_height = int(sqrt(self.max_output_pixels))
+            max_output_size = (output_width, output_height)
+
         doc = template.substitute(service=bunch(default='', **self.service),
                                    layers=self.layers,
                                    formats=self.image_formats,
@@ -529,6 +538,7 @@ class Capabilities(object):
                                    layer_srs_bbox=self.layer_srs_bbox,
                                    layer_llbbox=self.layer_llbbox,
                                    inspire_md=inspire_md,
+                                   max_output_size=max_output_size,
         )
         # strip blank lines
         doc = '\n'.join(l for l in doc.split('\n') if l.rstrip())
@@ -568,7 +578,7 @@ class LayerRenderer(object):
         render_layers = combined_layers(self.layers, self.query)
         if not render_layers: return
 
-        async_pool = async.Pool(size=min(len(render_layers), self.concurrent_rendering))
+        async_pool = async_.Pool(size=min(len(render_layers), self.concurrent_rendering))
 
         if self.raise_source_errors:
             return self._render_raise_exceptions(async_pool, render_layers, layer_merger)
